package com.sd.business.controller.inventory;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.NonInterception;
import com.sd.business.entity.in.dto.CorrectionDto;
import com.sd.business.entity.inventory.dto.GetQuantityDto;
import com.sd.business.entity.inventory.dto.InventorySelectDto;
import com.sd.business.entity.inventory.dto.QuantityByWarehouseDto;
import com.sd.business.entity.inventory.vo.InventoryVo;
import com.sd.business.entity.inventory.vo.QuantityByDepartmentVo;
import com.sd.business.entity.inventory.vo.QuantityByWarehouseVo;
import com.sd.business.service.inventory.InventoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 库存 前端控制器
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@RestController
@RequestMapping("/inventory")
public class InventoryController {

    @Autowired
    private InventoryService inventoryService;

    /**
     * 库存分页
     */
    @PostMapping("/page")
    public Page<InventoryVo> page(@RequestBody InventorySelectDto dto) {
        dto.setBackupDate(null);
        return inventoryService.getPage(dto, "inventory");
    }

    /**
     * 根据仓库获取库存数量
     */
    @PostMapping("/getQuantityByWarehouse")
    public List<QuantityByWarehouseVo> getQuantityByWarehouse(@RequestBody QuantityByWarehouseDto dto) {
        return inventoryService.getQuantityByWarehouse(dto);
    }

    /**
     * 根据事业部获取库存数量
     */
    @PostMapping("/getQuantityByDepartment")
    public List<QuantityByDepartmentVo> getQuantityByDepartment() {
        return inventoryService.getQuantityByDepartment();
    }

    /**
     * 查询库存
     */
    @PostMapping("/getQuantity")
    public Map<Long, BigDecimal> getQuantity(@RequestBody GetQuantityDto dto) {
        return inventoryService.getQuantity(dto);
    }

    /**
     * 库存导出
     */
    @NonInterception
    @GetMapping("/exportExcel")
    public void exportExcel(InventorySelectDto dto) {
        inventoryService.exportExcel(dto);
    }

    /**
     * 库存导出
     */
    @NonInterception
    @GetMapping("/exportBackExcel")
    public void exportBackExcel(InventorySelectDto dto) {
        System.err.println("dto.backupDate");
        inventoryService.exportBackExcel(dto);
    }

    /**
     * 新增库存
     */
    @PostMapping("/add")
    public void add(@RequestBody JSONObject dto) {
        inventoryService.add(dto.getString("str"), dto.getLong("warehouseId"), dto.getLong("departmentId"));
    }

    /**
     * 库存修正
     */
    @PostMapping("/correction")
    public void correction(@Validated @RequestBody CorrectionDto dto) {
        inventoryService.correction(dto);
    }

}
