package com.sd.business.controller.statement;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.NonInterception;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.sd.business.entity.statement.dto.FileUploadDto;
import com.sd.business.entity.statement.dto.StatementOfAccountDto;
import com.sd.business.entity.statement.dto.StatementOfAccountSelectDto;
import com.sd.business.entity.statement.vo.*;
import com.sd.business.service.statement.StatementOfAccountService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Collections;
import java.util.List;


/**
 * <p>
 * 对账单 前端控制器
 * </p>
 *
 * @author
 * @since 2023-07-31
 */
@RestController
@RequestMapping("/statementOfAccount")
public class StatementOfAccountController {

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    /**
     * 对账单分页
     */
    @PostMapping("/page")
    public Page<StatementOfAccountVo> page(@RequestBody StatementOfAccountSelectDto dto) {
        return statementOfAccountService.getPage(dto);
    }

    /**
     * 对账单明细
     */
    @PostMapping("/detail")
    public List<ReconciliationDetailVo> detail(@RequestBody BaseSelectDto dto) {
        return statementOfAccountService.detail(dto.getId());
    }

    /**
     * 对账单新增
     */
    @PostMapping("/add")
    public void add(@Validated @RequestBody StatementOfAccountDto statementOfAccountDto) {
        statementOfAccountService.add(statementOfAccountDto);
    }

    /**
     * 对账单编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody StatementOfAccountDto statementOfAccountDto) {
        statementOfAccountService.edit(statementOfAccountDto);
    }

    /**
     * 编辑对账单文件
     */
    @PostMapping("/editReceiptFile")
    public void editReceiptFile(@RequestBody FileUploadDto dto) {
        statementOfAccountService.editFile(dto, 1);
    }

    /**
     * 编辑转账凭证文件
     */
    @PostMapping("/editProofFile")
    public void editProofFile(@RequestBody FileUploadDto dto) {
        statementOfAccountService.editFile(dto, 2);
    }

    /**
     * 对账单删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        statementOfAccountService.delete(dto.getId());
    }

    /**
     * sku对账
     */
    @PostMapping("/getDocumentBySku")
    public List<DocumentBySkuVo> getDocumentBySku(@RequestBody BaseSelectDto dto) {
        return statementOfAccountService.getDocumentBySku(Collections.singletonList(dto.getId()));
    }

    /**
     * bom对账
     */
    @PostMapping("/getDocumentByBom")
    public List<DocumentByBomVo> getDocumentByBom(@RequestBody BaseSelectDto dto) {
        return statementOfAccountService.getDocumentByBom(Collections.singletonList(dto.getId()));
    }

    /**
     * 订单对账
     */
    @PostMapping("/getDocumentByOrder")
    public List<DocumentByOrderVo> getDocumentByOrder(@RequestBody BaseSelectDto dto) {
        return statementOfAccountService.getDocumentByOrder(Collections.singletonList(dto.getId()));
    }

    /**
     * 导出sku对账单
     */
    @NonInterception
    @GetMapping("/exportDocumentBySku")
    public void exportDocumentBySku(BaseSelectDto dto) {
        statementOfAccountService.export(dto.getId(), 1);
    }

    /**
     * 导出bom对账单
     */
    @NonInterception
    @GetMapping("/exportDocumentByBom")
    public void exportDocumentByBom(BaseSelectDto dto) {
        statementOfAccountService.export(dto.getId(), 2);
    }

    /**
     * 导出订单对账单
     */
    @NonInterception
    @GetMapping("/exportDocumentByOrder")
    public void exportDocumentByOrder(BaseSelectDto dto) {
        statementOfAccountService.export(dto.getId(), 3);
    }

    /**
     * 获取万里牛单号
     */
    @PostMapping("/getOrderWlnCodeStr")
    public String getOrderWlnCodeStr(@RequestBody List<Long> idList) {
        return statementOfAccountService.getOrderWlnCodeStr(idList);
    }

}
