package com.sd.business.service.inventory.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.framework.mybatis.holder.LogicHolder;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.in.dto.CorrectionDto;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.in.po.InOutStorageDetails;
import com.sd.business.entity.inventory.bo.InOutFun;
import com.sd.business.entity.inventory.dto.GetQuantityDto;
import com.sd.business.entity.inventory.dto.InventorySelectDto;
import com.sd.business.entity.inventory.dto.QuantityByWarehouseDto;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.inventory.po.InventoryBackup;
import com.sd.business.entity.inventory.vo.InventoryVo;
import com.sd.business.entity.inventory.vo.QuantityByDepartmentVo;
import com.sd.business.entity.inventory.vo.QuantityByWarehouseVo;
import com.sd.business.mapper.inventory.InventoryMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.in.InOutStorageDetailsService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.framework.util.excel.util.ExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 库存 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@Slf4j
@Service
public class InventoryServiceImpl extends ServiceImpl<InventoryMapper, Inventory> implements InventoryService {

    @Autowired
    private HttpServletResponse response;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private InOutStorageDetailsService inOutStorageDetailsService;

    @Override
    public Page<InventoryVo> getPage(InventorySelectDto dto, String tableName) {
        IWrapper<Inventory> wrapper = getDtoWrapper(dto);
        return this.baseMapper.getPage(dto.getPage(), wrapper, tableName);
    }

    @Override
    public List<QuantityByWarehouseVo> getQuantityByWarehouse(QuantityByWarehouseDto dto) {
        return baseMapper.getQuantityByWarehouse(dto);
    }

    @Override
    public List<QuantityByDepartmentVo> getQuantityByDepartment() {
        return baseMapper.getQuantityByDepartment();
    }

    @Override
    public void in(Long departmentId, Long warehouseId, List<? extends InOutFun> list) {
        List<Long> bomSpecIdList = list.stream().map(InOutFun::getBomSpecId).collect(Collectors.toList());

        synchronized (this) {

            // 通过事业部id和出库id获取bom规格库存
            List<Inventory> inventoryList = getInventoryList(departmentId, warehouseId, bomSpecIdList);

            // 通过bom规格分组
            Map<Long, Inventory> map = inventoryList.stream().collect(Collectors.toMap(Inventory::getBomSpecId, Function.identity()));

            // 保存结存单价批次
            List<InOutStorageDetails> inOutStorageDetailsList = list.stream().map(item -> {

                // todo 获取临时结存单价
                Inventory inventory = map.get(item.getBomSpecId());
                BigDecimal inUnitPrice = inventory == null ? BigDecimal.ZERO : inventory.getBalanceUnitPrice();

                InOutStorageDetails inOutStorageDetails = new InOutStorageDetails();
                inOutStorageDetails.setDepartmentId(departmentId);
                inOutStorageDetails.setWarehouseId(warehouseId);
                inOutStorageDetails.setBomSpecId(item.getBomSpecId());
                inOutStorageDetails.setInUnitPrice(inUnitPrice);
                inOutStorageDetails.setQuantity(item.getQuantity());
                return inOutStorageDetails;
            }).collect(Collectors.toList());
            inOutStorageDetailsService.saveBatch(inOutStorageDetailsList);

            // 通过出库id获取bom规格结存列表
            Map<Long, List<InOutStorageDetails>> inOutStorageDetailsMap = getInOutStorageDetailsMap(warehouseId, bomSpecIdList);


            for (InOutFun inOutFun : list) {

                // 库存
                Inventory inventory = map.computeIfAbsent(inOutFun.getBomSpecId(), item -> {
                    Inventory tempInventory = new Inventory();
                    tempInventory.setWarehouseId(warehouseId);
                    tempInventory.setDepartmentId(departmentId);
                    tempInventory.setBomSpecId(item);
                    tempInventory.setQuantity(BigDecimal.ZERO);
                    inventoryList.add(tempInventory);
                    return tempInventory;
                });

                // 获取入库结存单价
                BigDecimal balanceUnitPrice = getBalanceUnitPrice(inOutStorageDetailsMap, inOutFun);

                inventory.setQuantity(inventory.getQuantity().add(inOutFun.getQuantity()));
                inventory.setBalanceUnitPrice(balanceUnitPrice);
            }

            saveOrUpdateBatch(inventoryList);
        }
    }

    @Override
    public void out(Long departmentId, Long warehouseId, List<? extends InOutFun> list) {
        List<Long> bomSpecIdList = list.stream().map(InOutFun::getBomSpecId).collect(Collectors.toList());

        synchronized (this) {

            // 通过事业部id和出库id获取bom规格库存
            List<Inventory> inventoryList = getInventoryList(departmentId, warehouseId, bomSpecIdList);
            Map<Long, Inventory> inventoryMap = inventoryList.stream().collect(Collectors.toMap(Inventory::getBomSpecId, Function.identity()));

            // 获取bom规格id和结存明细列表map
            Map<Long, List<InOutStorageDetails>> inOutStorageDetailsMap = getInOutStorageDetailsMap(warehouseId, bomSpecIdList);

            for (InOutFun inOutFun : list) {
                Long bomSpecId = inOutFun.getBomSpecId();
                BigDecimal quantity = inOutFun.getQuantity();

                Inventory inventory = inventoryMap.get(bomSpecId);
                if (inventory == null || inventory.getQuantity().compareTo(quantity) < 0) {
                    BomSpec bomSpec = bomSpecService.getById(bomSpecId);
                    if (bomSpec == null) {
                        throw new ServiceException("未知bom规格id：" + bomSpecId);
                    }
                    throw new ServiceException("出库失败,品名为：" + bomSpec.getName() + " 的bom库存不足,库存为：" +
                            (inventory == null ? BigDecimal.ZERO : inventory.getQuantity()) + ",需要出库：" + quantity);
                }

                // 计算出库结存单价
                BigDecimal balanceUnitPrice = getOutBalanceUnitPrice(inOutStorageDetailsMap, inOutFun);

                // 赋值结存单价
                inventory.setBalanceUnitPrice(balanceUnitPrice);
                inventory.setQuantity(inventory.getQuantity().subtract(quantity));
            }

            // 修改计算结存单价的出入库明细表
            List<InOutStorageDetails> inOutStorageDetailsList = inOutStorageDetailsMap.values()
                    .stream().flatMap(Collection::stream).collect(Collectors.toList());
            inOutStorageDetailsService.updateBatchById(inOutStorageDetailsList);

            // 修改库存数量以及结存单价
            updateBatchById(inventoryList);
        }
    }

    @Override
    public Map<Long, BigDecimal> getQuantity(GetQuantityDto dto) {

        return mapKV(Inventory::getBomSpecId, Inventory::getQuantity, q -> q
                .eq(Inventory::getWarehouseId, dto.getWarehouseId())
                .eq(Inventory::getDepartmentId, dto.getDepartmentId())
                .in(Inventory::getBomSpecId, dto.getBomSpecIdList())
        );

    }

    @Override
    public void exportExcel(InventorySelectDto dto) {
        IWrapper<Inventory> wrapper = getDtoWrapper(dto);
        List<InventoryVo> list = baseMapper.getList(wrapper);
        ExcelUtil.export(response, DateUtil.format(new Date(), "yyyy-MM-dd库存数据"),
                "库存数据", list, InventoryVo.class);
    }

    @Override
    public void add(String str, Long warehouseId, Long departmentId) {
        List<List<String>> list = Arrays.stream(str.split("\n"))
                .map(item -> Arrays.stream(item.split("\t")).filter(StrUtil::isNotBlank).collect(Collectors.toList()))
                .collect(Collectors.toList());

        Map<String, Long> bomSpecMap = bomSpecService.mapKV(BomSpec::getCode, BaseIdPo::getId, null);

        for (List<String> data : list) {

            String code = data.get(0);
            String quantity = data.size() == 1 ? "0" : data.get(1);

            Long bomSpecId = bomSpecMap.get(code);

            boolean update = update(q -> q
                    .eq(Inventory::getBomSpecId, bomSpecId)
                    .eq(Inventory::getWarehouseId, warehouseId)
                    .eq(Inventory::getDepartmentId, departmentId)
                    .set(Inventory::getQuantity, quantity)
            );

            if (!update) {
                Inventory inventory = new Inventory();
                inventory.setBomSpecId(bomSpecId);
                inventory.setWarehouseId(warehouseId);
                inventory.setDepartmentId(departmentId);
                inventory.setQuantity(new BigDecimal(quantity));
                inventory.setBalanceUnitPrice(BigDecimal.ZERO);
                save(inventory);
            }

            boolean update1 = inOutStorageDetailsService.update(q -> q
                    .eq(InOutStorageDetails::getBomSpecId, bomSpecId)
                    .eq(InOutStorageDetails::getWarehouseId, warehouseId)
                    .eq(InOutStorageDetails::getDepartmentId, departmentId)
                    .set(InOutStorageDetails::getQuantity, quantity)
            );

            if (!update1) {
                InOutStorageDetails inOutStorageDetails = new InOutStorageDetails();
                inOutStorageDetails.setBomSpecId(bomSpecId);
                inOutStorageDetails.setWarehouseId(warehouseId);
                inOutStorageDetails.setDepartmentId(departmentId);
                inOutStorageDetails.setQuantity(new BigDecimal(quantity));
                inOutStorageDetails.setInUnitPrice(BigDecimal.ZERO);
                inOutStorageDetailsService.save(inOutStorageDetails);
            }

        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void correction(CorrectionDto dto) {
        List<InOutStorageBom> inOutStorageBomList = dto.getInOutStorageBomList();
        List<Long> bomSpecIdList = inOutStorageBomList.stream().map(InOutStorageBom::getBomSpecId).collect(Collectors.toList());

        List<Inventory> addInventoryList = new ArrayList<>();
        List<Inventory> updateInventoryList;
        List<InOutStorageDetails> addInOutStorageDetailsList = new ArrayList<>();

        synchronized (this) {

            updateInventoryList = getInventoryList(dto.getDepartmentId(), dto.getWarehouseId(), bomSpecIdList);
            Map<Long, Inventory> map = updateInventoryList.stream().collect(Collectors.toMap(Inventory::getBomSpecId, Function.identity()));

            for (InOutStorageBom inOutStorageBom : inOutStorageBomList) {
                Long bomSpecId = inOutStorageBom.getBomSpecId();
                BigDecimal quantity = inOutStorageBom.getQuantity();
                Inventory inventory = map.get(bomSpecId);

                InOutStorageDetails inOutStorageDetails = new InOutStorageDetails();
                inOutStorageDetails.setWarehouseId(dto.getWarehouseId());
                inOutStorageDetails.setDepartmentId(dto.getDepartmentId());
                inOutStorageDetails.setBomSpecId(bomSpecId);
                inOutStorageDetails.setQuantity(quantity);

                if (inventory == null) {
                    Inventory addInventory = new Inventory();
                    addInventory.setWarehouseId(dto.getWarehouseId());
                    addInventory.setDepartmentId(dto.getDepartmentId());
                    addInventory.setBomSpecId(bomSpecId);
                    addInventory.setQuantity(quantity);
                    addInventory.setBalanceUnitPrice(BigDecimal.ZERO);
                    addInventoryList.add(addInventory);

                    inOutStorageDetails.setInUnitPrice(BigDecimal.ZERO);
                } else {
                    inventory.setQuantity(quantity);

                    inOutStorageDetails.setInUnitPrice(inventory.getBalanceUnitPrice());
                }

                addInOutStorageDetailsList.add(inOutStorageDetails);
            }

            LogicHolder.setLogicHolder(true);
            inOutStorageDetailsService.remove(q -> q
                    .eq(InOutStorageDetails::getWarehouseId, dto.getWarehouseId())
                    .eq(InOutStorageDetails::getDepartmentId, dto.getDepartmentId())
                    .in(InOutStorageDetails::getBomSpecId, bomSpecIdList)
            );
            LogicHolder.clear();

            inOutStorageDetailsService.saveBatch(addInOutStorageDetailsList);

            if (addInventoryList.size() > 0) {
                saveBatch(addInventoryList);
            }

            if (updateInventoryList.size() > 0) {
                updateBatchById(updateInventoryList);
            }

        }

    }

    @Override
    public void exportBackExcel(InventorySelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(999999999);
        IWrapper<Inventory> wrapper = getDtoWrapper(dto);
        Page<InventoryVo> inventoryBackup = baseMapper.getPage(dto.getPage(), wrapper, "inventory_backup");
        List<InventoryVo> list = inventoryBackup.getRecords();
        ExcelUtil.export(response, DateUtil.format(dto.getBackupDate(), "yyyy-MM-dd库存数据"),
                "库存数据", list, InventoryVo.class);
    }

    /**
     * 通过事业部id和出库id获取bom规格库存
     */
    private List<Inventory> getInventoryList(Long departmentId, Long warehouseId, List<Long> bomSpecIdList) {
        return list(q -> q
                .eq(Inventory::getDepartmentId, departmentId)
                .eq(Inventory::getWarehouseId, warehouseId)
                .in(Inventory::getBomSpecId, bomSpecIdList)
        );
    }

    /**
     * 通过出库id获取bom规格结存列表
     */
    private Map<Long, List<InOutStorageDetails>> getInOutStorageDetailsMap(Long warehouseId, List<Long> bomSpecIdList) {
        List<InOutStorageDetails> list = inOutStorageDetailsService.list(q -> q
                .eq(InOutStorageDetails::getWarehouseId, warehouseId)
                .in(InOutStorageDetails::getBomSpecId, bomSpecIdList)
                .gt(InOutStorageDetails::getQuantity, BigDecimal.ZERO)
        );
        return list.stream().collect(Collectors.groupingBy(InOutStorageDetails::getBomSpecId));
    }

    /**
     * 获取入库结存单价
     */
    private BigDecimal getBalanceUnitPrice(Map<Long, List<InOutStorageDetails>> inOutStorageDetailsMap, InOutFun inOutFun) {

        // 获取结存数量
        BigDecimal sumQuantity = inOutStorageDetailsMap
                .getOrDefault(inOutFun.getBomSpecId(), Collections.emptyList())
                .stream()
                .map(InOutStorageDetails::getQuantity)
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        // 获取结存金额
        BigDecimal sumPrice = inOutStorageDetailsMap
                .getOrDefault(inOutFun.getBomSpecId(), Collections.emptyList())
                .stream()
                .map(item -> item.getQuantity().multiply(item.getInUnitPrice()))
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        // 结存单价
        if (sumQuantity.compareTo(BigDecimal.ZERO) == 0) {
            return BigDecimal.ZERO;
        }

        // 计算新结存单价
        return sumPrice.divide(sumQuantity, 2, RoundingMode.HALF_UP);
    }

    /**
     * 获取出库结存单价
     */
    private BigDecimal getOutBalanceUnitPrice(Map<Long, List<InOutStorageDetails>> inOutStorageDetailsMap, InOutFun inOutFun) {

        Long bomSpecId = inOutFun.getBomSpecId();
        BigDecimal quantity = inOutFun.getQuantity();

        List<InOutStorageDetails> outStorageDetails = inOutStorageDetailsMap.getOrDefault(bomSpecId, Collections.emptyList());

        // 循环次数
        int j = 0;

        do {

            if (outStorageDetails.size() == j) {
                throw new ServiceException("数据异常，无法计算库存单价");
            }

            // 获取这条记录剩余的数量
            InOutStorageDetails inOutStorageDetails = outStorageDetails.get(j);

            // 单条可出库数量
            BigDecimal tempQuantity = inOutStorageDetails.getQuantity();

            if (tempQuantity.compareTo(quantity) >= 0) {
                inOutStorageDetails.setQuantity(tempQuantity.subtract(quantity));
                quantity = BigDecimal.ZERO;
            } else {
                inOutStorageDetails.setQuantity(BigDecimal.ZERO);
                quantity = quantity.subtract(tempQuantity);
            }

            j++;

        } while (quantity.compareTo(BigDecimal.ZERO) > 0);

        return getBalanceUnitPrice(inOutStorageDetailsMap, inOutFun);
    }

    /**
     * 获取查询条件
     */
    private IWrapper<Inventory> getDtoWrapper(InventorySelectDto dto) {
        IWrapper<Inventory> wrapper = getWrapper();

        wrapper.orderByDesc("i", Inventory::getId);
        wrapper.eq("b", Bom::getBomClassifyId, dto.getBomClassifyId());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.eq("b", Bom::getSpecies, dto.getBomSpecies());
        wrapper.eq("b", Bom::getChromatophore, dto.getBomChromatophore());
        wrapper.eq("b", Bom::getEmbossingProcess, dto.getBomEmbossingProcess());
        wrapper.eq("b", Bom::getFrontGrain, dto.getBomFrontGrain());
        wrapper.eq("b", Bom::getReverseGrain, dto.getBomReverseGrain());
        wrapper.like("bs", BomSpec::getColour, dto.getBomSpecColour());
        wrapper.like("bs", BomSpec::getLength, dto.getBomSpecLength());
        wrapper.like("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("i", Inventory::getWarehouseId, dto.getWarehouseId());
        wrapper.eq("i", Inventory::getDepartmentId, dto.getDepartmentId());
        wrapper.eq("i", InventoryBackup::getBackupDate, dto.getBackupDate());

        return wrapper;
    }

}
