package com.sd.business.service.order.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.entity.ObsFile;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.annotation.LogicIgnore;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.service.ISysUserService;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.order.dto.*;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.*;
import com.sd.business.entity.order.vo.*;
import com.sd.business.entity.price.po.PriceBillingStandard;
import com.sd.business.entity.price.po.PriceBillingStandardDetail;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.statement.po.StatementOfAccount;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.order.OrderMapper;
import com.sd.business.service.bom.BomService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.order.*;
import com.sd.business.service.price.PriceBillingStandardDetailService;
import com.sd.business.service.price.PriceBillingStandardService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 订单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, OrderInfo> implements OrderService {

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private BomService bomService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Autowired
    private OrderOperatingLogService orderOperatingLogService;

    @Autowired
    private PriceBillingStandardService priceBillingStandardService;

    @Autowired
    private PriceBillingStandardDetailService priceBillingStandardDetailService;

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Autowired
    private ISysUserService sysUserService;

    @Override
    public Page<OrderInfoVo> getPage(OrderSelectDto dto) {
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.orderByDesc("o", OrderInfo::getId);
        wrapper.eq("d", Department::getId, dto.getDepartmentId());
        wrapper.like("d", Department::getName, dto.getDepartmentName());
        wrapper.like("o", OrderInfo::getCode, dto.getCode());
        wrapper.like("o", OrderInfo::getWlnCode, dto.getWlnCode());
        wrapper.eq("o", OrderInfo::getStatus, dto.getStatus());
        wrapper.eq("o", OrderInfo::getSettlementStatus, dto.getSettlementStatus());
        wrapper.ge("o", OrderInfo::getWlnCreateTime, dto.getBeginTime());
        wrapper.le("o", OrderInfo::getWlnCreateTime, dto.getEndTime());
        wrapper.eq("o", OrderInfo::getFlowStatus, dto.getFlowStatus());
        wrapper.and(q -> q
                .and(r -> r.eq("o", OrderInfo::getWlnStorageCode, "T007").ge("o", OrderInfo::getWlnStatus, 2))
                .or(r -> r.ne("o", OrderInfo::getWlnStorageCode, "T007").ge("o", OrderInfo::getWlnStatus, 1))
                .or(r -> r.eq("o", OrderInfo::getSource, 1))
        );

        List<Long> detpIdList = sysUserService.getDetpIdListByUserId(SecurityUtils.getUserId());
        if (detpIdList.size() > 0) {
            wrapper.in("o", OrderInfo::getDepartmentId, detpIdList);
            wrapper.eq("o", OrderInfo::getCreateUser, SecurityUtils.getUserId());
        }

        if (Objects.equals(dto.getLinkedStatementOfAccount(), 0)) {
            wrapper.isNull("o.statement_of_account_id");
            wrapper.in("o", OrderInfo::getStatus,
                    OrderStatusEnum.COMPLETION_PRODUCTION.getKey(),
                    OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        } else if (Objects.equals(dto.getLinkedStatementOfAccount(), 1)) {
            wrapper.isNotNull("o.statement_of_account_id");
            wrapper.in("o", OrderInfo::getStatus,
                    OrderStatusEnum.COMPLETION_PRODUCTION.getKey(),
                    OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        }

        if (dto.getException() != null) {
            if (dto.getException().equals(StatusConstant.YES)) {
                wrapper.ne("o", OrderInfo::getExceptionType, "0");
            } else {
                wrapper.eq("o", OrderInfo::getExceptionType, "0");
            }
        }

        wrapper.apply(dto.getExceptionType() != null,
                "find_in_set({0}, o.exception_type)", dto.getExceptionType());

        Page<OrderInfoVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<OrderInfoVo> records = page.getRecords();

        if (records.size() == 0) {
            return page;
        }

        for (OrderInfoVo record : records) {

            // 设置异常提示
            String exceptionType = record.getExceptionType();
            if (exceptionType.equals("0")) {
                continue;
            }
            String exceptionTypeDetail = Arrays.stream(exceptionType.split(","))
                    .map(Convert::toInt)
                    .map(OrderExceptionTypeEnum::getEnumValue)
                    .collect(Collectors.joining(","));
            record.setExceptionTypeDetail(exceptionTypeDetail);
        }

        return page;
    }

    @Override
    public OrderInfoVo detail(Long id) {
        OrderInfo orderInfo = this.getById(id);
        if (orderInfo == null) {
            throw new ServiceException("未知订单");
        }

        // 订单
        OrderInfoVo result = BeanUtil.toBean(orderInfo, OrderInfoVo.class);

        // 赋值事业部名称
        Department department = departmentService.getById(orderInfo.getDepartmentId());
        if (department != null) {
            result.setDepartmentName(department.getName());
        }

        // 赋值订单明细
        List<OrderSku> tempOrderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuVo> orderSkuList = BeanUtil.copyToList(tempOrderSkuList, OrderSkuVo.class);

        skuSpecService.attributeAssign(orderSkuList, OrderSkuVo::getSkuSpecId, (item, skuSpec) -> {
            item.setName(skuSpec.getName());
            item.setCode(skuSpec.getCode());
        });
        bomSpecService.attributeAssign(orderSkuList, OrderSkuVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomName(bomSpec.getName());
            item.setBomCode(bomSpec.getCode());
        });
        result.setOrderSkuList(orderSkuList);

        // 赋值包材配件/单品
        List<OrderSkuBom> tempOrderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));
        List<OrderSkuBomVo> orderSkuBomList = BeanUtil.copyToList(tempOrderSkuBomList, OrderSkuBomVo.class);
        bomSpecService.attributeAssign(orderSkuBomList, OrderSkuBom::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });
        Map<Long, List<OrderSkuBomVo>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        for (OrderSkuVo orderSkuVo : orderSkuList) {
            orderSkuVo.setOrderSkuBomList(orderSkuBomMap.getOrDefault(orderSkuVo.getId(), Collections.emptyList()));
        }

        // 赋值包装明细
        List<OrderPackageBom> list = orderPackageBomService.list(q -> q.eq(OrderPackageBom::getOrderId, id));

        List<OrderInfoVo.OrderPackage> orderPackageList = list.stream().map(item -> {
            OrderInfoVo.OrderPackage orderPackage = new OrderInfoVo.OrderPackage();
            orderPackage.setBomSpecId(item.getBomSpecId());
            orderPackage.setQuantity(item.getQuantity());
            orderPackage.setCostPrice(item.getCostPrice());
            orderPackage.setInternalSellingPrice(item.getInternalSellingPrice());
            return orderPackage;
        }).collect(Collectors.toList());
        // 赋值品名 品号 销售单价 成本价
        bomSpecService.attributeAssign(orderPackageList, OrderInfoVo.OrderPackage::getBomSpecId, (item, bomSpec) -> {
            item.setCode(bomSpec.getCode());
            item.setName(bomSpec.getName());
        });
        result.setOrderPackageBomList(orderPackageList);

        return result;
    }

    @DSTransactional
    @Override
    public void add(OrderInfoDto orderDto) {
        orderDto.setSource(1);
        orderDto.setCode("PI" + new Date().getTime());
        orderDto.setSettlementStatus(1);
        orderDto.setTaxRate(new BigDecimal("10.00"));
        orderDto.setTag("0");
        orderDto.setExceptionType(OrderExceptionTypeEnum.NORMAL.getKey().toString());
        this.save(orderDto);

        List<OrderSkuDto> orderSkuList = orderDto.getOrderSkuList();
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(orderDto.getId()))
                .peek(item -> item.setStockPreparationStatus(StatusConstant.NO))
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.saveBatch(tempOrderSkuList);

        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.saveFile(Collections.singletonList(item.getSelfAdhesiveStickerFile()), item.getId());
        }

        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(orderDto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.saveBatch(orderSkuBomList);

        if (ObjectUtil.isNotEmpty(orderDto.getOrderPackageBomList())) {
            // 保存订单包装信息
            List<OrderPackageBomDto> tempOrderPackageBomList = orderDto.getOrderPackageBomList();
            List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                    .peek(item -> item.setOrderId(orderDto.getId()))
                    .map(item -> (OrderPackageBom) item)
                    .collect(Collectors.toList());
            orderPackageBomService.saveBatch(orderPackageBomList);
        }

        OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
        orderOperatingLog.setType(10);
        orderOperatingLog.setOrderId(orderDto.getId());
        orderOperatingLog.setOrderCode(orderDto.getCode());
        orderOperatingLogService.save(orderOperatingLog);

        ObsFileUtil.saveFile(orderDto.getFileList(), orderDto.getId());

        // 保存 外箱不干胶图稿, 文件业务类型为 1
        ObsFile outerBoxSelfAdhesiveStickerFile = orderDto.getOuterBoxSelfAdhesiveStickerFile();
        if (ObjectUtil.isNotEmpty(outerBoxSelfAdhesiveStickerFile)) {
            ObsFileUtil.saveFile(Collections.singletonList(outerBoxSelfAdhesiveStickerFile), orderDto.getId(), 1);
        }

    }

    @DSTransactional
    @Override
    public void edit(OrderInfoDto orderDto) {
        this.updateById(orderDto);

        List<OrderSkuDto> orderSkuList = orderDto.getOrderSkuList();
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(orderDto.getId()))
                .peek(item -> item.setStockPreparationStatus(StatusConstant.NO))
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.editLinked(tempOrderSkuList, OrderSku::getOrderId, orderDto.getId());

        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.saveFile(Collections.singletonList(item.getSelfAdhesiveStickerFile()), item.getId());
        }

        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(orderDto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.editLinked(orderSkuBomList, OrderSkuBom::getOrderId, orderDto.getId());

        if (ObjectUtil.isNotEmpty(orderDto.getOrderPackageBomList())) {
            // 更新订单包装信息
            List<OrderPackageBomDto> tempOrderPackageBomList = orderDto.getOrderPackageBomList();
            List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                    .peek(item -> item.setOrderId(orderDto.getId()))
                    .map(item -> (OrderPackageBom) item)
                    .collect(Collectors.toList());
            orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, orderDto.getId());
        }

        if (orderDto.getUpdateType() != null) {
            OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
            orderOperatingLog.setType(orderDto.getUpdateType());
            orderOperatingLog.setOrderId(orderDto.getId());
            orderOperatingLog.setOrderCode(orderDto.getCode());
            orderOperatingLogService.save(orderOperatingLog);
        }

        ObsFileUtil.editFile(orderDto.getFileList(), orderDto.getId());

        // 更新 外箱不干胶图稿
        ObsFile outerBoxSelfAdhesiveStickerFile = orderDto.getOuterBoxSelfAdhesiveStickerFile();
        if (ObjectUtil.isNotEmpty(outerBoxSelfAdhesiveStickerFile)) {
            ObsFileUtil.editFile(Collections.singletonList(outerBoxSelfAdhesiveStickerFile), orderDto.getId(), 1);
        }

    }

    @DSTransactional
    @Override
    public void delete(Long id, boolean saveLog) {
        OrderInfo orderInfo = getById(id);

        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }

        if (Objects.equals(orderInfo.getDelFlag(), StatusConstant.NOT_DELETED)
                && orderInfo.getStatus() > OrderStatusEnum.STOCK_PREPARATION.getKey()) {
            throw new ServiceException("订单正在生产中，无法直接删除，需要走审批流程");
        }

        // 删除订单以及订单关联数据
        delete(id);

        if (saveLog) {
            OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
            orderOperatingLog.setType(30);
            orderOperatingLog.setOrderId(id);
            orderOperatingLog.setOrderCode(orderInfo.getCode());
            orderOperatingLogService.save(orderOperatingLog);
        }

    }

    @Override
    public SkuSpecPriceVo getSkuSpecPrice(SkuSpecPriceDto dto) {

        // 查询单价
        Long skuSpecId = dto.getSkuSpecId();
        SkuSpec skuSpec = skuSpecService.getById(skuSpecId);
        Assert.notNull(skuSpec, "未知sku规格");
        Long bomSpecId = skuSpec.getBomSpecId();
        Assert.notNull(bomSpecId, "sku未绑定bom");
        BomSpec bomSpec = bomSpecService.getById(bomSpecId);
        Assert.notNull(bomSpec, "未知bom规格");
        Bom bom = bomService.getById(bomSpec.getBomId());
        Assert.notNull(bom, "未知bom");

        // 其他价格查询
        Long deptId = SecurityUtils.getDeptId();
        Department department = departmentService.getById(deptId);
        Assert.notNull(department, "未知事业部");
        String priceBillingStandardIdStr = department.getPriceBillingStandardId();
        Assert.notBlank(priceBillingStandardIdStr, "事业部未绑定加工计费标准");

        List<String> priceBillingStandardIdList = Arrays.stream(priceBillingStandardIdStr.split(",")).collect(Collectors.toList());
        List<PriceBillingStandard> priceBillingStandardList = priceBillingStandardService.listByIds(priceBillingStandardIdList);

        List<PriceBillingStandardDetail> priceBillingStandardDetailList = null;

        if (StrUtil.isBlank(bom.getSpecies())) {
            priceBillingStandardDetailList = Collections.emptyList();
        } else {
            for (PriceBillingStandard priceBillingStandard : priceBillingStandardList) {
                if (Objects.equals(bom.getSpecies(), priceBillingStandard.getSpecies())) {
                    priceBillingStandardDetailList = priceBillingStandardDetailService.list(
                            q -> q.eq(PriceBillingStandardDetail::getPriceBillingStandardId, priceBillingStandard.getId()));
                    break;
                }
            }

            if (priceBillingStandardDetailList == null) {
                throw new ServiceException("没有找到加工计费标准");
            }
        }

        // 赋值
        SkuSpecPriceVo skuSpecPriceVo = new SkuSpecPriceVo();
        skuSpecPriceVo.setUnitPrice(bomSpec.getInternalSellingPrice());
        skuSpecPriceVo.setCustomProcessingFee(BigDecimal.ZERO);
        skuSpecPriceVo.setLssueFee(BigDecimal.ZERO);
        skuSpecPriceVo.setDeliveryMaterialsFee(BigDecimal.ZERO);
        skuSpecPriceVo.setPackingLabor(BigDecimal.ZERO);
        skuSpecPriceVo.setManagementFee(BigDecimal.ZERO);

        for (PriceBillingStandardDetail priceBillingStandardDetail : priceBillingStandardDetailList) {
            String chargeItem = priceBillingStandardDetail.getChargeItem();
            BigDecimal chargePrice = ObjectUtil.defaultIfNull(priceBillingStandardDetail.getChargePrice(), BigDecimal.ZERO);
            switch (chargeItem) {
                case "40":
                    skuSpecPriceVo.setPackingLabor(chargePrice);
                    break;
                case "50":
                    skuSpecPriceVo.setLssueFee(chargePrice);
                    break;
                case "60":
                    skuSpecPriceVo.setDeliveryMaterialsFee(chargePrice);
                    break;
                case "70":
                    skuSpecPriceVo.setManagementFee(chargePrice);
                    break;
                default:
                    if (Objects.equals(chargeItem, skuSpec.getMachinedPanel())
                            && dto.getQuantity().compareTo(priceBillingStandardDetail.getSectionMin()) >= 0
                            && dto.getQuantity().compareTo(priceBillingStandardDetail.getSectionMax()) < 0) {
                        skuSpecPriceVo.setCustomProcessingFee(chargePrice);
                        skuSpecPriceVo.setCustomProcessingType(chargeItem);
                    }
            }
        }

        return skuSpecPriceVo;
    }

    @Override
    public String addExceptionType(String exceptionTypeStr, Integer addExceptionType) {
        if (StrUtil.isBlank(exceptionTypeStr) || exceptionTypeStr.equals("0")) {
            return addExceptionType.toString();
        }

        if (Arrays.asList(exceptionTypeStr.split(",")).contains(addExceptionType.toString())) {
            return exceptionTypeStr;
        }

        return exceptionTypeStr + "," + addExceptionType;
    }

    @LogicIgnore
    @Override
    public Page<OrderInfoVo> deletedOrderPage(OrderSelectDto dto) {
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.orderByDesc("o", OrderInfo::getId);
        wrapper.eq("d", Department::getId, dto.getDepartmentId());
        wrapper.like("o", OrderInfo::getCode, dto.getCode());
        wrapper.like("o", OrderInfo::getWlnCode, dto.getWlnCode());
        wrapper.eq("o", OrderInfo::getDelFlag, StatusConstant.DELETED);

        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @LogicIgnore
    @Override
    public OrderInfoVo deletedOrderDetail(BaseSelectDto dto) {
        return detail(dto.getId());
    }

    @Override
    public void turnToRegularOrder(Long id) {
        OrderInfo orderInfo = getById(id);
        String exceptionType = orderInfo.getExceptionType();

        if (Objects.equals(exceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString())) {
            return;
        }

        for (String s : exceptionType.split(",")) {
            OrderExceptionTypeEnum orderExceptionTypeEnum = OrderExceptionTypeEnum.getEnum(Convert.toInt(s));
            if (ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_CLOSURE)
                    && ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_ABORT)
                    && ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_EXCEPTION_HANDLING)) {
                throw new ServiceException(orderExceptionTypeEnum.getValue() + "，无法转为正常订单");
            }
        }

        update(q -> q
                .eq(BaseIdPo::getId, orderInfo.getId())
                .set(OrderInfo::getExceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

    }

    @DSTransactional
    @Override
    public synchronized void deleteAndStore(Long id) {

        OrderInfo orderInfo = getById(id);

        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }

        if (orderInfo.getStatus() <= OrderStatusEnum.STOCK_PREPARATION.getKey()) {
            return;
        }

        // 获取订单商品主材和包材
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));

        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));

        // 合并主材和包材
        List<InOutStorageBom> inOutStorageBomList = orderSkuList.stream().map(item -> {
            InOutStorageBom inOutStorageBom = new InOutStorageBom();
            inOutStorageBom.setBomSpecId(item.getBomSpecId());
            inOutStorageBom.setQuantity(item.getQuantity());
            return inOutStorageBom;
        }).collect(Collectors.toList());

        for (OrderSkuBom orderSkuBom : orderSkuBomList) {
            OrderSku orderSku = orderSkuMap.get(orderSkuBom.getOrderSkuId());
            InOutStorageBom inOutStorageBom = new InOutStorageBom();
            inOutStorageBom.setBomSpecId(orderSkuBom.getBomSpecId());
            inOutStorageBom.setQuantity(orderSkuBom.getQuantity().multiply(orderSku.getQuantity()));
            inOutStorageBomList.add(inOutStorageBom);
        }

        // 合并数量
        Map<Long, InOutStorageBom> map = inOutStorageBomList.stream().collect(Collectors.toMap(
                InOutStorageBom::getBomSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                }
        ));
        inOutStorageBomList = new ArrayList<>(map.values());

        // 区分半成品仓和包材仓库
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(map.keySet());

        // 主材从半成品仓入库
        List<InOutStorageBom> semiFinishedProductInOutStorageBomList = inOutStorageBomList
                .stream()
                .filter(item -> {
                    BomSpecBo tempBomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (tempBomSpecBo == null) {
                        return false;
                    }
                    return ObjectUtil.equal(tempBomSpecBo.getClassifyParentId(), 1L);
                })
                .collect(Collectors.toList());
        InOutStorageDto semiFinishedProductInOutStorageDto = new InOutStorageDto();
        semiFinishedProductInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
        semiFinishedProductInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
        semiFinishedProductInOutStorageDto.setWarehouseId(WarehouseConstant.SEMI_FINISHED_PRODUCT);
        semiFinishedProductInOutStorageDto.setDepartmentId(orderInfo.getOutDepartmentId());
        semiFinishedProductInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        semiFinishedProductInOutStorageDto.setRemark("删除订单：" + orderInfo.getCode() + " 主材退料入库");
        semiFinishedProductInOutStorageDto.setInOutStorageBomList(semiFinishedProductInOutStorageBomList);
        inOutStorageService.add(semiFinishedProductInOutStorageDto);

        // 包材从包材仓出库
        List<InOutStorageBom> packagingMaterialInOutStorageBomList = inOutStorageBomList
                .stream()
                .filter(item -> {
                    BomSpecBo tempBomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (tempBomSpecBo == null) {
                        return true;
                    }
                    return ObjectUtil.notEqual(tempBomSpecBo.getClassifyParentId(), 1L);
                })
                .collect(Collectors.toList());
        InOutStorageDto packagingMaterialInOutStorageDto = new InOutStorageDto();
        packagingMaterialInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
        packagingMaterialInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
        packagingMaterialInOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
        packagingMaterialInOutStorageDto.setDepartmentId(orderInfo.getOutDepartmentId());
        packagingMaterialInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        packagingMaterialInOutStorageDto.setRemark("删除订单：" + orderInfo.getCode() + " 包材退料入库");
        packagingMaterialInOutStorageDto.setInOutStorageBomList(packagingMaterialInOutStorageBomList);
        inOutStorageService.add(packagingMaterialInOutStorageDto);

        delete(id);
    }

    @Override
    public CompareVo compare(OrderSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(999999999);
        Page<OrderInfoVo> page = getPage(dto);
        List<OrderInfoVo> records = page.getRecords();
        List<String> wlnCodeList = records.stream().map(OrderInfo::getWlnCode).collect(Collectors.toList());

        String compareStr = dto.getCompareStr();
        List<String> list = Arrays.asList(compareStr.split("\n"));

        List<String> systemMissingOrderList = list.stream().filter(item -> !wlnCodeList.contains(item)).collect(Collectors.toList());
        List<String> systemSurplusOrderList = wlnCodeList.stream().filter(item -> !list.contains(item)).collect(Collectors.toList());

        CompareVo compareVo = new CompareVo();
        compareVo.setSystemMissingOrderList(systemMissingOrderList);
        compareVo.setSystemSurplusOrderList(systemSurplusOrderList);

        return compareVo;
    }

    @Override
    public BigDecimal getOrderBomSpecOutStorageQuantity(Long bomSpecId, Integer days) {
        Date date = DateUtil.date();
        Date offsetDay = DateUtil.offsetDay(date, -days);

        // 根据对账时间获取有对账单的订单
        List<StatementOfAccount> list = statementOfAccountService.list(q -> q.ge(StatementOfAccount::getTimePeriod, offsetDay));
        List<Long> accountIds = list.stream().map(StatementOfAccount::getId).collect(Collectors.toList());

        Map<Long, BomSpecBo> bomSpecBo = skuSpecService.getBomSpecBoByIdList(Collections.singleton(bomSpecId));
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.ge("o", OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey());
        wrapper.in("o", OrderInfo::getStatementOfAccountId, accountIds);
        if (ObjectUtil.equal(bomSpecBo.get(bomSpecId).getClassifyParentId(), 1L)) {
            // 查询主材消耗数量
            wrapper.eq("os", OrderSku::getBomSpecId, bomSpecId);
            return this.baseMapper.getOrderSkuOutStorageQuantity(wrapper);
        } else {
            // 查询包材消耗数量
            wrapper.eq("osb", OrderSkuBom::getBomSpecId, bomSpecId);
            return this.baseMapper.getOrderSkuBomOutStorageQuantity(wrapper);
        }
    }

    @Override
    public String getOrderWlnCodeStr(OrderSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(999999999);
        Page<OrderInfoVo> page = getPage(dto);
        List<String> wlnCodeList = page.getRecords().stream().map(OrderInfo::getWlnCode).collect(Collectors.toList());
        return JSON.toJSONString(wlnCodeList);
    }

    @Override
    public Boolean isExistOrderSkuBom(Long orderId) {
        OrderInfo orderInfo = this.getById(orderId);
        if (!ObjectUtil.equals(orderInfo.getType(), 2)) {
            return false;
        }
        boolean flag = true;
        List<OrderSku> list = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderInfo.getId()));
        for (OrderSku orderSku : list) {
            long count = orderSkuBomService.count(q -> q.eq(OrderSkuBom::getOrderSkuId, orderSku.getId()));
            if (count != 0) {
                flag = false;
            }
        }
        return flag;
    }

    /**
     * 删除订单以及订单关联数据
     */
    private void delete(Long id) {
        this.removeById(id);
        orderSkuService.remove(q -> q.eq(OrderSku::getOrderId, id));
        orderSkuBomService.remove(q -> q.eq(OrderSkuBom::getOrderId, id));
        orderPackageBomService.remove(q -> q.eq(OrderPackageBom::getOrderId, id));

        ObsFileUtil.removeFile(id);
    }

}
