package com.sd.business.service.order.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.dto.OrderSkuSelectDto;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.vo.OrderSkuVo;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.order.OrderSkuMapper;
import com.sd.business.service.order.OrderSkuService;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * 订单sku 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-06
 */
@Service
public class OrderSkuServiceImpl extends ServiceImpl<OrderSkuMapper, OrderSku> implements OrderSkuService {

    @Override
    public Page<OrderSkuVo> getPage(OrderSkuSelectDto dto) {
        IWrapper<OrderSku> wrapper = getWrapper();
        wrapper.orderByDesc("os", OrderSku::getId);
        wrapper.like("oi", OrderInfo::getCode, dto.getOrderCode());
        wrapper.like("oi", OrderInfo::getWlnCode, dto.getOrderWlnCode());
        wrapper.like("d", Department::getName, dto.getDepartmentName());
        wrapper.like("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.like("ss", SkuSpec::getName, dto.getSkuSpecName());
        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.eq("oi", OrderInfo::getStatus, dto.getOrderStatus());
        wrapper.eq("oi", OrderInfo::getDepartmentId, dto.getDepartmentId());
        wrapper.ge("oi", OrderInfo::getWlnCreateTime, dto.getBeginTime());
        wrapper.le("oi", OrderInfo::getWlnCreateTime, dto.getEndTime());
        wrapper.eq("oi", OrderInfo::getExceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString());
        wrapper.and(q -> q
                .and(r -> r.eq("oi", OrderInfo::getWlnStorageCode, "T007").ge("oi", OrderInfo::getWlnStatus, 2))
                .or(r -> r.ne("oi", OrderInfo::getWlnStorageCode, "T007").ge("oi", OrderInfo::getWlnStatus, 1))
                .or(r -> r.eq("oi", OrderInfo::getSource, 1))
        );

        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Override
    public String getOrderWlnCodeStr(OrderSkuSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<OrderSkuVo> page = getPage(dto);
        List<OrderSkuVo> records = page.getRecords();

        List<String> wlnCodeList = records.stream().map(OrderSkuVo::getOrderCode).filter(ObjectUtil::isNotNull).collect(Collectors.toList());
        return JSON.toJSONString(wlnCodeList);
    }

}
