package com.sd.business.service.packaging.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.bom.vo.BomSpecVo;
import com.sd.business.entity.packaging.dto.PackagingAssemblyDto;
import com.sd.business.entity.packaging.dto.PackagingAssemblySelectDto;
import com.sd.business.entity.packaging.po.PackagingAssembly;
import com.sd.business.entity.packaging.po.PackagingAssemblyBom;
import com.sd.business.entity.packaging.vo.PackagingAssemblyVo;
import com.sd.business.mapper.packaging.PackagingAssemblyMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.packaging.PackagingAssemblyBomService;
import com.sd.business.service.packaging.PackagingAssemblyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class PackagingAssemblyServiceImpl extends ServiceImpl<PackagingAssemblyMapper, PackagingAssembly> implements PackagingAssemblyService {

    @Autowired
    private PackagingAssemblyBomService packagingAssemblyBomService;

    @Autowired
    private BomSpecService bomSpecService;

    @Override
    public Page<PackagingAssemblyVo> getPage(PackagingAssemblySelectDto dto) {
        IWrapper<PackagingAssembly> wrapper = getWrapper();
        wrapper.orderByDesc("pa", PackagingAssembly::getId);
        wrapper.like("pa", PackagingAssembly::getName, dto.getName());
        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Override
    public void add(PackagingAssemblyDto dto) {
        this.save(dto);
        // 保存包装组合bom关联信息
        List<PackagingAssemblyBom> packagingAssemblyBomList = dto.getPackagingAssemblyBomList();
        packagingAssemblyBomList.forEach(q -> q.setPackagingAssemblyId(dto.getId()));
        packagingAssemblyBomService.saveBatch(packagingAssemblyBomList);
    }

    @Override
    public void edit(PackagingAssemblyDto dto) {

    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
        packagingAssemblyBomService.remove(q -> q.eq(PackagingAssemblyBom::getPackagingAssemblyId, id));
    }

    @Override
    public List<BomSpecVo> getPackagingBomSpecList(BaseSelectDto dto) {
        List<PackagingAssemblyBom> list = packagingAssemblyBomService.list(q -> q.eq(PackagingAssemblyBom::getPackagingAssemblyId, dto.getId()));
        List<Long> bomSpecIdList = list.stream().map(PackagingAssemblyBom::getBomSpecId).collect(Collectors.toList());
        List<BomSpec> bomSpecs = bomSpecService.list(q -> q.in(BomSpec::getId, bomSpecIdList));
        return BeanUtil.copyToList(bomSpecs, BomSpecVo.class);
    }
}
