package com.sd.business.service.statement.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.entity.FileInfoVo;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.framework.config.ThreadPoolConfig;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuBom;
import com.sd.business.entity.statement.dto.FileUploadDto;
import com.sd.business.entity.statement.dto.StatementOfAccountDto;
import com.sd.business.entity.statement.dto.StatementOfAccountSelectDto;
import com.sd.business.entity.statement.po.StatementOfAccount;
import com.sd.business.entity.statement.vo.*;
import com.sd.business.mapper.statement.StatementOfAccountMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuBomService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountExportService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.Comparator.comparing;


/**
 * <p>
 * 对账单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-31
 */
@Service
public class StatementOfAccountServiceImpl extends ServiceImpl<StatementOfAccountMapper, StatementOfAccount> implements StatementOfAccountService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private StatementOfAccountExportService statementOfAccountExportService;

    @Qualifier(ThreadPoolConfig.threadPoolTaskExecutor)
    @Autowired
    private ThreadPoolExecutor threadPoolExecutor;

    @Override
    public Page<StatementOfAccountVo> getPage(StatementOfAccountSelectDto dto) {

        IWrapper<StatementOfAccount> wrapper = getWrapper();
        wrapper.like("soa", StatementOfAccount::getCode, dto.getCode());
        wrapper.eq("soa", StatementOfAccount::getDepartmentId, dto.getDepartmentId());
        wrapper.ge("soa", StatementOfAccount::getTimePeriod, dto.getBeginTime());
        wrapper.le("soa", StatementOfAccount::getTimePeriod, dto.getEndTime());
        wrapper.orderByDesc("soa", StatementOfAccount::getCode);

        Page<StatementOfAccountVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<StatementOfAccountVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        // 查询对账单和转帐凭证
        List<Long> idList = records.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        Map<Long, List<FileInfoVo>> receiptFileMap = ObsFileUtil.getFileMap(idList, 1);
        Map<Long, List<FileInfoVo>> proofFileMap = ObsFileUtil.getFileMap(idList, 2);

        Map<Long, Map<String, Object>> map = orderService.listMaps(Wrappers.<OrderInfo>query()
                        .select(
                                "statement_of_account_id as statementOfAccountId",
                                "sum(total_amount) as amount",
                                "count(0) as orderNum"
                        )
                        .in("statement_of_account_id", idList)
                        .groupBy("statement_of_account_id")
                )
                .stream()
                .collect(Collectors.toMap(
                        item -> Convert.toLong(item.get("statementOfAccountId")),
                        Function.identity()
                ));

        for (StatementOfAccountVo record : records) {
            record.setReceiptFileList(receiptFileMap.getOrDefault(record.getId(), Collections.emptyList()));
            record.setProofFileList(proofFileMap.getOrDefault(record.getId(), Collections.emptyList()));
            Map<String, Object> tempMap = map.get(record.getId());
            if (tempMap == null) {
                record.setOrderNum(0);
                record.setAmount(BigDecimal.ZERO);
            } else {
                record.setOrderNum(Convert.toInt(tempMap.get("orderNum"), 0));
                record.setAmount(Convert.toBigDecimal(tempMap.get("amount"), BigDecimal.ZERO));
            }
        }

        return page;
    }

    @Override
    public List<ReconciliationDetailVo> detail(Long id) {

        List<OrderInfo> orderInfoList = orderService.list(q -> q.eq(OrderInfo::getStatementOfAccountId, id));

        List<ReconciliationDetailVo> result = orderInfoList.stream().map(item -> ReconciliationDetailVo.builder()
                .orderId(item.getId())
                .departmentId(item.getDepartmentId())
                .orderCode(item.getCode())
                .totalAmount(item.getTotalAmount())
                .productTotalAmount(item.getProductTotalAmount())
                .customProcessingFee(item.getCustomProcessingFee())
                .lssueFee(item.getLssueFee())
                .deliveryMaterialsFee(item.getDeliveryMaterialsFee())
                .packingLabor(item.getPackingLabor())
                .packagingMaterialCost(item.getPackagingMaterialCost())
                .managementFee(item.getManagementFee())
                .outerBoxPackingFee(item.getOuterBoxPackingFee())
                .build()
        ).collect(Collectors.toList());

        departmentService.attributeAssign(result, ReconciliationDetailVo::getDepartmentId, (item, department) -> {
            item.setDepartmentName(department.getName());
        });

        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(StatementOfAccountDto statementOfAccountDto) {

        List<Long> orderIdList = statementOfAccountDto.getOrderIdList();

        statementOfAccountDto.setCode(CodeEnum.STATEMENT_OF_ACCOUNT_CODE.getCode());
        statementOfAccountDto.setType(1);
        statementOfAccountDto.setTimePeriod(new Date());
        this.save(statementOfAccountDto);

        if (ObjectUtil.isNotEmpty(orderIdList)) {
            orderService.update(q -> q
                    .in(BaseIdPo::getId, orderIdList)
                    .set(OrderInfo::getStatementOfAccountId, statementOfAccountDto.getId())
            );
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(StatementOfAccountDto statementOfAccountDto) {
        Long statementOfAccountId = statementOfAccountDto.getId();
        List<Long> orderIdList = statementOfAccountDto.getOrderIdList();

        Assert.notNull(statementOfAccountId, "对账单id不能为空");

        if (ObjectUtil.isNotEmpty(orderIdList)) {
            orderService.update(q -> q
                    .in(BaseIdPo::getId, orderIdList)
                    .set(OrderInfo::getStatementOfAccountId, statementOfAccountId)
            );
        }

        orderService.update(q -> q
                .eq(OrderInfo::getStatementOfAccountId, statementOfAccountId)
                .notIn(ObjectUtil.isNotEmpty(orderIdList), BaseIdPo::getId, orderIdList)
                .set(OrderInfo::getStatementOfAccountId, null)
        );

    }

    @DSTransactional
    @Override
    public void delete(Long id) {

        orderService.update(q -> q
                .eq(OrderInfo::getStatementOfAccountId, id)
                .set(OrderInfo::getStatementOfAccountId, null)
        );

        this.removeById(id);
        ObsFileUtil.removeFile(id);
    }

    @Override
    public void editFile(FileUploadDto dto, int type) {
        ObsFileUtil.editFile(dto.getFileList(), dto.getId(), type);
    }

    @Override
    public List<DocumentBySkuVo> getDocumentBySku(List<Long> statementOfAccountIdList) {

        // 获取订单id列表
        List<Long> orderIdList = getOrderIdList(statementOfAccountIdList);

        if (orderIdList.size() == 0) {
            return Collections.emptyList();
        }

        // 获取订单sku
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));

        // 生成结果集
        List<DocumentBySkuVo> documentBySkuVoList = orderSkuList.stream()
                .map(item -> DocumentBySkuVo.builder()
                        .skuSpecId(item.getSkuSpecId())
                        .orderId(item.getOrderId())
                        // 判断是否是委外订单，如果是委外订单时将bom数量修改为0
                        .quantity(item.getQuantity())
                        .unitPrice(item.getUnitPrice()
                                .add(item.getCustomProcessingFee())
                                .add(item.getLssueFee())
                                .add(item.getDeliveryMaterialsFee())
                                .add(item.getPackingLabor())
                                .add(item.getPackagingMaterialCost())
                                .add(item.getManagementFee())
                        )
                        .build())
                .peek(item -> item.setSubtotal(ObjectUtil.equals(item.getQuantity(), BigDecimal.ZERO) ? item.getUnitPrice() : item.getQuantity().multiply(item.getUnitPrice())))
                .peek(item -> item.setTotal(item.getSubtotal()))
                .peek(item -> {
                    // 判断是否是委外订单，如果是委外订单时将bom数量修改为0
                    OrderInfo orderInfo = orderService.getOne(q -> q.eq(OrderInfo::getId, item.getOrderId()).eq(OrderInfo::getType, 2));
                    if (ObjectUtil.isNotEmpty(orderInfo)) {
                        item.setQuantity(BigDecimal.ZERO);
                    }
                })
                .collect(Collectors.toList());

        // 赋值sku规格品名和品号
        skuSpecService.attributeAssign(documentBySkuVoList, DocumentBySkuVo::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        // 合并单价、小计、sku规格id系统的对账数据
        Collection<DocumentBySkuVo> documentBySkuVoCollection = documentBySkuVoList.stream()
                .sorted(comparing(DocumentBySkuVo::getSkuSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getSkuSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getTotal().add(v2.getTotal()));
                            return v1;
                        }
                )).values();

        return new ArrayList<>(documentBySkuVoCollection);
    }

    @Override
    public List<DocumentByBomVo> getDocumentByBom(List<Long> statementOfAccountIdList) {

        // 获取订单id列表
        List<Long> orderIdList = getOrderIdList(statementOfAccountIdList);

        if (orderIdList.size() == 0) {
            return Collections.emptyList();
        }

        // 获取订单sku
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));

        // 主材bom
        List<DocumentByBomVo> result = orderSkuList.stream()
                .map(item -> DocumentByBomVo.builder()
                                .bomSpecId(item.getBomSpecId())
                                .orderId(item.getOrderId())
                                .quantity(item.getQuantity())
                                .unitPrice(item.getUnitPrice())
                                .laserLogoSummary((Objects.equals(item.getCustomProcessingType(), "20")
                                        ? item.getCustomProcessingFee().multiply(item.getQuantity()) : BigDecimal.ZERO))
                                .laserMitochondrialSummary((Objects.equals(item.getCustomProcessingType(), "10")
                                        ? item.getCustomProcessingFee().multiply(item.getQuantity()) : BigDecimal.ZERO))
                                .lssueFeeSummary(item.getLssueFee().multiply(item.getQuantity()))
                                .deliveryMaterialsFeeSummary(item.getDeliveryMaterialsFee().multiply(item.getQuantity()))
                                .packingLaborSummary(item.getPackingLabor().multiply(item.getQuantity()))
                                .managementFeeSummary(item.getManagementFee().multiply(item.getQuantity()))
                                .build())
                .peek(item -> item.setSubtotal(
                        item.getUnitPrice()
                                .multiply(item.getQuantity())
                                .add(item.getLaserLogoSummary())
                                .add(item.getLaserMitochondrialSummary())
                                .add(item.getLssueFeeSummary())
                                .add(item.getDeliveryMaterialsFeeSummary())
                                .add(item.getPackingLaborSummary())
                                .add(item.getManagementFeeSummary())
                ))
                .peek(item -> item.setTotal(item.getSubtotal()))
                .peek(item -> {
                    // 判断是否是委外订单，如果是委外订单时将bom数量修改为0
                    OrderInfo orderInfo = orderService.getOne(q -> q.eq(OrderInfo::getId, item.getOrderId()).eq(OrderInfo::getType, 2));
                    if (ObjectUtil.isNotEmpty(orderInfo)) {
                        item.setQuantity(BigDecimal.ZERO);
                    }
                })
                .collect(Collectors.toList());

        // 赋值主材bom品名品号
        bomSpecService.attributeAssign(result, DocumentByBomVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        // 合并同加个同规格主材bom，数量相加
        Collection<DocumentByBomVo> documentByBomVoCollection = result.stream()
                .sorted(comparing(DocumentByBomVo::getBomSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getBomSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setLaserLogoSummary(v1.getLaserLogoSummary().add(v2.getLaserLogoSummary()));
                            v1.setLaserMitochondrialSummary(v1.getLaserMitochondrialSummary().add(v2.getLaserMitochondrialSummary()));
                            v1.setLssueFeeSummary(v1.getLssueFeeSummary().add(v2.getLssueFeeSummary()));
                            v1.setDeliveryMaterialsFeeSummary(v1.getDeliveryMaterialsFeeSummary().add(v2.getDeliveryMaterialsFeeSummary()));
                            v1.setPackingLaborSummary(v1.getPackingLaborSummary().add(v2.getPackingLaborSummary()));
                            v1.setManagementFeeSummary(v1.getManagementFeeSummary().add(v2.getManagementFeeSummary()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getSubtotal());
                            return v1;
                        }
                )).values();

        // 赋值合并后结果
        result = new ArrayList<>(documentByBomVoCollection);
        // 按品名升序排序
        result.sort(comparing(DocumentByBomVo::getBomSpecName));

        Map<Long, BigDecimal> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, OrderSku::getQuantity));

        // 包材bom
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderId, orderIdList));
        List<DocumentByBomVo> bomVoList = orderSkuBomList.stream().map(item ->
                DocumentByBomVo.builder()
                        .bomSpecId(item.getBomSpecId())
                        .quantity(item.getQuantity().multiply(orderSkuMap.get(item.getOrderSkuId())))
                        .unitPrice(item.getUnitPrice())
                        .subtotal(item.getUnitPrice().multiply(item.getQuantity().multiply(orderSkuMap.get(item.getOrderSkuId()))))
                        .total(item.getUnitPrice().multiply(item.getQuantity().multiply(orderSkuMap.get(item.getOrderSkuId()))))
                        .build()
        ).collect(Collectors.toList());

        // 赋值包材bom品名品号
        bomSpecService.attributeAssign(bomVoList, DocumentByBomVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        // 合并同加个同规格包材bom，数量相加
        Collection<DocumentByBomVo> bomValues = bomVoList.stream()
                .sorted(comparing(DocumentByBomVo::getBomSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getBomSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getSubtotal());
                            return v1;
                        }
                )).values();

        // 合并主材包材
        result.addAll(bomValues);

        return result;
    }

    @Override
    public List<DocumentByOrderVo> getDocumentByOrder(List<Long> statementOfAccountId) {

        Assert.notEmpty(statementOfAccountId, "对账单id不能为空");
        List<OrderInfo> orderList = orderService.list(q -> q.in(OrderInfo::getStatementOfAccountId, statementOfAccountId));
        if (orderList.size() == 0) {
            return Collections.emptyList();
        }

        List<DocumentByOrderVo> result = orderList.stream().map(item ->
                DocumentByOrderVo.builder()
                        .orderId(item.getId())
                        .wlnCreateTime(ObjectUtil.defaultIfNull(item.getWlnCreateTime(), item.getCreateTime()))
                        .code(item.getCode())
                        .wlnCode(item.getWlnCode())
                        .outerBoxPackingFee(item.getOuterBoxPackingFee())
                        .total(item.getTotalAmount())
                        .build()
        ).collect(Collectors.toList());

        List<Long> orderIdList = orderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));

        SecurityContext context = SecurityContextHolder.getContext();

        CompletableFuture<List<DocumentByOrderVo.SkuSpec>> skuSpecListCompletableFuture = CompletableFuture.supplyAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    return getSkuSpecList(orderSkuList);
                }, threadPoolExecutor);

        CompletableFuture<List<DocumentByOrderVo.BomSpec>> bomSpecListCompletableFuture = CompletableFuture.supplyAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    return getBomSpecList(orderIdList, orderSkuList);
                }, threadPoolExecutor);

        // 获取订单bom
        List<DocumentByOrderVo.BomSpec> bomSpecList = bomSpecListCompletableFuture.join();

        // sku赋值bom
        Map<Long, List<DocumentByOrderVo.BomSpec>> bomSpecMap = bomSpecList.stream()
                .collect(Collectors.groupingBy(DocumentByOrderVo.BomSpec::getOrderSkuId));

        // 获取订单sku
        List<DocumentByOrderVo.SkuSpec> skuSpecList = skuSpecListCompletableFuture.join();

        for (DocumentByOrderVo.SkuSpec skuSpec : skuSpecList) {
            skuSpec.setBomSpecList(bomSpecMap.getOrDefault(skuSpec.getOrderSkuId(), Collections.emptyList()));
        }

        // 订单赋值sku
        Map<Long, List<DocumentByOrderVo.SkuSpec>> skuSpecMap = skuSpecList.stream()
                .collect(Collectors.groupingBy(DocumentByOrderVo.SkuSpec::getOrderId));
        for (DocumentByOrderVo documentByOrderVo : result) {
            documentByOrderVo.setSkuSpecList(skuSpecMap.getOrDefault(documentByOrderVo.getOrderId(), Collections.emptyList()));
        }

        return result;
    }

    @Override
    public void export(Long id, int type) {

        StatementOfAccount statementOfAccount = getById(id);
        Assert.notNull(statementOfAccount, "没有找到对账单");

        Department department = departmentService.getById(statementOfAccount.getDepartmentId());
        Assert.notNull(department, "没有找到事业部");

        Date timePeriod = statementOfAccount.getTimePeriod();
        String date = timePeriod != null ? DateUtil.formatDate(timePeriod) : StringPool.EMPTY;
        String departmentName = department.getName();

        List<Long> idList = Collections.singletonList(id);

        switch (type) {
            case 1:
                statementOfAccountExportService.exportDocumentBySku(idList, departmentName, date, date);
                break;
            case 2:
                statementOfAccountExportService.exportDocumentByBom(idList, departmentName, date, date);
                break;
            case 3:
                statementOfAccountExportService.exportDocumentByOrder(idList, departmentName, date, date);
                break;
            default:
                throw new ServiceException("未知对账单类型");
        }

    }

    @Override
    public String getOrderWlnCodeStr(List<Long> idList) {
        List<OrderInfo> list = orderService.list(q -> q.in(OrderInfo::getStatementOfAccountId, idList));
        List<String> wlnCodeList = list.stream().map(OrderInfo::getWlnCode).filter(ObjectUtil::isNotNull).collect(Collectors.toList());
        return JSON.toJSONString(wlnCodeList);
    }

    /**
     * 根据对账单id获取对账订单id列表
     */
    private List<Long> getOrderIdList(List<Long> statementOfAccountIdList) {
        Assert.notEmpty(statementOfAccountIdList, "对账单id不能为空");
        List<OrderInfo> orderList = orderService.list(q -> q.in(OrderInfo::getStatementOfAccountId, statementOfAccountIdList));
        return orderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
    }

    /**
     * 获取订单sku
     */
    private List<DocumentByOrderVo.SkuSpec> getSkuSpecList(List<OrderSku> orderSkuList) {

        List<DocumentByOrderVo.SkuSpec> skuSpecList = orderSkuList.stream()
                .map(item -> DocumentByOrderVo.SkuSpec.builder()
                        .orderId(item.getOrderId())
                        .orderSkuId(item.getId())
                        .skuSpecId(item.getSkuSpecId())
                        // 判断是否是委外订单，如果是委外订单时将bom数量修改为0
                        .quantity(item.getQuantity())
                        .unitPrice(item.getUnitPrice()
                                .add(item.getCustomProcessingFee())
                                .add(item.getLssueFee())
                                .add(item.getDeliveryMaterialsFee())
                                .add(item.getPackingLabor())
                                .add(item.getPackagingMaterialCost())
                                .add(item.getManagementFee()))
                        .build())
                .peek(item -> item.setSubtotal(item.getQuantity().multiply(item.getUnitPrice())))
                .peek(item -> {
                    // 判断是否是委外订单，如果是委外订单时将bom数量修改为0
                    OrderInfo orderInfo = orderService.getOne(q -> q.eq(OrderInfo::getId, item.getOrderId()).eq(OrderInfo::getType, 2));
                    if (ObjectUtil.isNotEmpty(orderInfo)) {
                        item.setQuantity(BigDecimal.ZERO);
                    }
                })
                .collect(Collectors.toList());

        skuSpecService.attributeAssign(skuSpecList, DocumentByOrderVo.SkuSpec::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        return skuSpecList;
    }

    /**
     * 获取订单bom
     */
    private List<DocumentByOrderVo.BomSpec> getBomSpecList(List<Long> orderIdList, List<OrderSku> orderSkuList) {

        List<DocumentByOrderVo.BomSpec> bomSpecList = orderSkuList.stream()
                .map(item -> DocumentByOrderVo.BomSpec.builder()
                        .orderSkuId(item.getId())
                        .bomSpecId(item.getBomSpecId())
                        // 判断是否是委外订单，如果是委外订单时将bom数量修改为0
                        .quantity(ObjectUtil.isNotEmpty(
                                orderService.getOne(q -> q
                                        .eq(OrderInfo::getId, item.getOrderId()).eq(OrderInfo::getType, 2)))
                                ? BigDecimal.ZERO : item.getQuantity())
                        .unitPrice(item.getUnitPrice())
                        .laserLogoSummary((Objects.equals(item.getCustomProcessingType(), "20")
                                ? item.getCustomProcessingFee() : BigDecimal.ZERO))
                        .laserMitochondrialSummary((Objects.equals(item.getCustomProcessingType(), "10")
                                ? item.getCustomProcessingFee() : BigDecimal.ZERO))
                        .lssueFeeSummary(item.getLssueFee())
                        .deliveryMaterialsFeeSummary(item.getDeliveryMaterialsFee())
                        .packingLaborSummary(item.getPackingLabor())
                        .managementFeeSummary(item.getManagementFee())
                        .build())
                .collect(Collectors.toList());

        Map<Long, BigDecimal> map = bomSpecList.stream().collect(
                Collectors.toMap(DocumentByOrderVo.BomSpec::getOrderSkuId, DocumentByOrderVo.BomSpec::getQuantity));

        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderId, orderIdList));

        List<DocumentByOrderVo.BomSpec> packBomSpecList = orderSkuBomList.stream()
                .map(item -> DocumentByOrderVo.BomSpec.builder()
                        .orderSkuId(item.getOrderSkuId())
                        .bomSpecId(item.getBomSpecId())
                        .quantity(item.getQuantity())
                        .unitPrice(item.getUnitPrice())
                        .build())
                .peek(item -> item.setQuantity(ObjectUtil.equals(map.get(item.getOrderSkuId()), BigDecimal.ZERO) ? item.getQuantity() : map.get(item.getOrderSkuId()).multiply(item.getQuantity())))
                .collect(Collectors.toList());

        bomSpecList.addAll(packBomSpecList);

        bomSpecService.attributeAssign(bomSpecList, DocumentByOrderVo.BomSpec::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        return bomSpecList;
    }

}
